
/* Citadel/UX message routines
 * 
 * contents:
 * 
 * read_message() - called by Read routines to print messages
 * send_message() - read message in temp file & store in msgmain
 * make_message() - assemble temp message file - msg editor
 * msgform()      - msg formatter / called by read_message() and others
 * pull_message() - pull message out of msgmain into temp file
 */

#include <fcntl.h>
#include <stdio.h>
#include <ctype.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <pwd.h>
#include <setjmp.h>
#include <termio.h>
#include "citadel.h"

#define IFEXPERT if (usersupp.flags&US_EXPERT)
#define IFNEXPERT if ((usersupp.flags&US_EXPERT)==0)
#define IFUPASS if (upass!=0)
#define IFNUPASS if (upass==0)
#define IFAIDE if (usersupp.axlevel>=6)
#define IFNAIDE if (usersupp.axlevel<6)
long atol();
long lseek();

extern int curr_rm;
extern struct usersupp usersupp;
extern struct quickroom quickroom;
extern jmp_buf nextbuf;
extern int (*backnext())();
extern int (*backstop())();
extern char *months[];

read_message(pos)	/* Read a message from the master file */
long pos;			/* Position in master file */
{
	int a;
	a=setjmp(nextbuf);
	sttybbs(0);
	if ((a==1)||(a==2)) {
		printf("\n\n");
		return(a);
		}
	signal(SIGINT,(*backnext));
	signal(SIGQUIT,(*backstop));
	sttybbs(1);
	msgform(usersupp.screenwidth,pos,"msgmain");
	sttybbs(0);
	return(0);
}

send_message(filename,retbuf)	/* send a message to the master file */
char filename[];		/* tempfilename of proper message */
struct smreturn *retbuf;	/* return information */
{

int file,a,c,d,file2;
long bb,cc,templen,hibytes,origpos;
char sbuf[BUFSIZ];
char mid[30];
FILE *fp;
struct msgmain msgmain;

	file=open("MMstructure",O_RDWR);
	if (file<0) interr(13);

	do {
		lseek(file,0L,0);
		a=read(file,&msgmain,sizeof(struct msgmain));
		if (a<1) interr(14);
		} while(msgmain.MMflags & MM_BUSY);

	lseek(file,0L,0);
	msgmain.MMflags=(msgmain.MMflags|MM_BUSY);
	a=write(file,&msgmain,sizeof(struct msgmain)); if (a<1) interr(16);
	close(file);
	origpos=msgmain.MMcurpos;
	++msgmain.MMhighest;
	sprintf(mid,"I%ld",msgmain.MMhighest);
/* measure the message */
file=open(filename,O_RDONLY); if (file<0) interr(17);
templen=lseek(file,0L,2);
close(file);
templen=templen+(long)strlen(mid);
++templen;

/* check for FF bytes */
hibytes=0L;
fp=fopen("msgmain","rb");
if (fp==NULL) interr(18);
bb=fseek(fp,msgmain.MMcurpos,0);

cc=ftell(fp);
for (bb=0L; bb<templen; ++bb) {
	if (cc>=MM_FILELEN) cc=fseek(fp,0L,0);
	c=getc(fp); ++cc;
	if (c>127) ++hibytes;		/* bump count if hi bit set */
	}

fclose(fp);
msgmain.MMlowest=msgmain.MMlowest+hibytes;

file=open("msgmain",O_RDWR);
cc=lseek(file,msgmain.MMcurpos,0);
file2=open(filename,O_RDONLY); if (file2<0) interr(19);
cc=lseek(file,0L,1);
d=read(file2,sbuf,3);
write(file,sbuf,d);
write(file,mid,strlen(mid));
d=0; write(file,&d,1);
do {
	d=read(file2,sbuf,BUFSIZ);
	write(file,sbuf,d);
	cc=cc+(long)d;
	if (((long)cc)>=((long)MM_FILELEN)) cc=lseek(file,0L,0);
	} while(d>0);

msgmain.MMcurpos=lseek(file,0L,1);
close(file2);	/* done with temp file */
close(file);	/* done with master file */

/* now update the message structure */

msgmain.MMflags=msgmain.MMflags & ~MM_BUSY;

file=open("MMstructure",O_RDWR); if (file<0) interr(20);
a=write(file,&msgmain,sizeof(struct msgmain)); if (a<1) interr(21);
close(file);

retbuf->smnumber=msgmain.MMhighest;
retbuf->smpos=origpos;

return(0);
}

make_message(filename,author,recipient,room,type,mode)
char filename[];	/* temporary file name */
struct usersupp *author;  /* author's usersupp structure */
char recipient[];	/* NULL if it's not mail */
char room[];		/* room where it's going */
int type;		/* see MES_ types in header file */
int mode;		/* 0 for normal, 1 for ASCII */
{ 
	FILE *fp;
	int a,b,old;
	long aa,beg,now;
	char aaa[100],bbb[100]; 

	time(&now);
	fp=fopen(filename,"wb"); if (fp==NULL) interr(22);
	putc(255,fp);
	putc(type,fp);	/* Normal or anonymous, see MES_ flags */
	putc(mode,fp);	/* Formatted or unformatted */
	strcpy(aaa,author->fullname);
	for (a=0; a<strlen(aaa); ++a) {
		aaa[a]=tolower(aaa[a]);
		if (aaa[a]==32) aaa[a]='_';
		if (aaa[a]=='!') strcpy(&aaa[a],&aaa[a+1]);
		}
	fprintf(fp,"P%s",aaa); putc(0,fp);			/* path */
	fprintf(fp,"T%ld",now); putc(0,fp);			/* date/time */
	fprintf(fp,"A%s",author->fullname); putc(0,fp);		/* author */
	fprintf(fp,"O%s",quickroom.QRname); putc(0,fp);		/* room */
	fprintf(fp,"N%s",NODENAME); putc(0,fp);			/* node */
	if (recipient[0]!=0) { fprintf(fp,"R%s",recipient); putc(0,fp); }
	fflush(fp);
	msgform(usersupp.screenwidth,0L,filename);
	putc('M',fp);
	if (mode==1) printf("(Press ctrl-d when finished)\n");
	beg=ftell(fp); old=1;
	if (mode==1) goto ME2;
ME1:	a=inkey(); a=(a&127);
	aa=ftell(fp);
	if ((a==8)&&(aa>beg)) {
		back(1); fseek(fp,-1L,1); goto ME1; }
	if ((a<32)&&(a!=13)) goto ME1;
	putc(a,stdout);
	if ((old==13)&&(a==13)) goto MECR;
	putc(a,fp); if (a==13) putc(10,stdout);
	old=a;
	goto ME1;

ME2:	a=inkey(); if (a==255) a=32;
	if (a==4) goto MECR;
	putc(a,fp); putc(a,stdout);
	if (a==13) putc(10,stdout);
	goto ME2;

MECR:	putc(0,fp); fseek(fp,-1L,1); fflush(fp);
	printf("<A>bort <C>ontinue <P>rint <S>ave -> ");
MECR2:	b=inkey(); b=(b&127); b=tolower(b);
	if (b=='a') { printf("Abort\n");	goto MEABT; }
	if (b=='c') { printf("Continue\n");
		if (mode==0) goto ME1;
		if (mode==1) goto ME2; }
	if (b=='s') { printf("Save buffer\n"); goto MEFIN; } 
	if (b=='p') { printf("Print formatted\n");
		msgform(usersupp.screenwidth,0L,filename);
		goto MECR; }
	goto MECR2;

MEFIN:	putc(0,fp);
	fclose(fp);
	return(0);

MEABT:	printf("Are you sure? ");
	if (yesno()==0) goto ME1;
	fseek(fp,0L,0);
	putc(0,fp);
	putc(0,fp);
	fclose(fp);
	return(2);

}

msgform(width,spos,msgfile)
int width;
long spos;
char *msgfile;
	{
	struct tm *tm;
	int a,b,c,e,real,old,mtype,aflag;
	char aaa[100],bbb[100],ttm[3],ap;
	FILE *fp;
	long now;

	fp=fopen(msgfile,"rb");
	if (fp==NULL) return(1);
	strcpy(aaa,""); old=255;
	c=1; /* current column pos */
	fseek(fp,spos,0);
	e=getc(fp);
	if (e!=255) {
		printf("*** CANNOT LOCATE MESSAGE / pos=%ld\n",spos);
		goto END;
		}
	mtype=getc(fp); aflag=getc(fp);
	printf("\n   ");
	if (mtype==MES_ANON) printf("****\n");
	if (mtype==MES_AN2) printf("<anonymous> ");
	goto BONFGM;
A:	if (aflag==1) goto AFLAG;
	old=real; a=getc(fp); real=a;
	if (a==0) goto END;
	if (a<0) goto END;
	
	if ( ((a==13)||(a==10)) && (old!=13) && (old!=10) ) a=32;
	if ( ((old==13)||(old==10)) && ((real==32)||(real==13)||(real==10))) {
						printf("\n"); c=1; }
	if (a!=32) {
	if ( ((strlen(aaa)+c)>(width-4)) && (strlen(aaa)>(width-4)) )
		{ printf("\n%s",aaa); c=strlen(aaa); aaa[0]=0; }
	 b=strlen(aaa); aaa[b]=a; aaa[b+1]=0; }
	if (a==32) { 	if ((strlen(aaa)+c)>(width-5)) { 
							printf("\n");
							c=1;
							}
			printf("%s ",aaa); ++c; c=c+strlen(aaa);
			strcpy(aaa,""); goto A; }
	if ((a==13)||(a==10)) {
				printf("%s\n",aaa); c=1;
				strcpy(aaa,""); goto A; }
	goto A;

AFLAG:	a=getc(fp); if (a==0) goto END;
	if ((a<32)&&(a!=13)&&(a!=10)) a='.';
	putc(a,stdout); if (a==13) putc(10,stdout);
	goto AFLAG;

END:	fclose(fp);
	printf("\n");
	return(0);

BONFGM:	b=getc(fp); if (b<0) goto END;
	if (b=='M') {
		printf("\n");
		goto A;
		}
	fpgetfield(fp,bbb);
	if ( (usersupp.axlevel<6)
		&& (usersupp.eternal!=quickroom.QRroomaide) )
			if ((mtype==MES_ANON)||(mtype==MES_AN2)) goto BONFGM;
	if (b=='A') printf("from %s ",bbb);
#ifdef SEEMID
	if ((b=='I')&&(usersupp.axlevel>=6)) printf("#%s ",bbb);
#endif		/* define SEEMID if you wish to see message ID's in header */

	if ((b=='N')&&(strucmp(bbb,NODENAME))&&(curr_rm==1))
		printf("@ %s ",bbb);
	if ((b=='O')&&(strucmp(bbb,quickroom.QRname))) printf("in %s> ",bbb);
	if ((b=='N')&&(quickroom.QRflags&QR_NETWORK))
		printf("@ %s ",bbb);
	if (b=='R') printf("to %s ",bbb);
	if (b=='T') {
		now=atol(bbb);
		tm=(struct tm *)localtime(&now);
		sprintf(ttm,"%2d",tm->tm_min);
		if (ttm[0]==32) ttm[0]=48;
		ap='a'; if ((tm->tm_hour)>12) {
			tm->tm_hour=((tm->tm_hour)-12);
			ap='p';
			}
		printf("%d-%s-%4d %d:%2s%cm ",
			tm->tm_mday,months[tm->tm_mon],
			(1900+tm->tm_year),
			tm->tm_hour,ttm,ap);
		}
	goto BONFGM;
}

pull_message(spos,flnm)		/* Pull a message out of the master file */
long spos;			/* position in master file */
char *flnm;			/* file to write to */
{
	int a,b,c;
	int file,file2;
	long cc;
	char field[512];
	char fwork[512];

	file2=creat(flnm,0666);
	file=open("msgmain",O_RDONLY);
	lseek(file,spos,0);

	for (a=0; a<3; ++a) {
		read(file,&b,1);
		write(file2,&b,1);
		}
PI:	b=0; read(file,&b,1);
	if (b=='M') {
		write(file2,&b,1);
		goto B;
		}
	getfield(file,field);
	if (b!='I') {
		write(file2,&b,1);
		write(file2,field,strlen(field)+1);
		}
	goto PI;

B:	cc=lseek(file,0L,1);
C:	if (cc>=MM_FILELEN) cc=lseek(file,0L,0);
	c=0; read(file,&c,1); ++cc;
	write(file2,&c,1);
	if (c!=0) goto C;

	close(file); close(file2);
	return(0);
}
